@@ -1,3 +1,5 @@
+import datetime
+from datetime import timedelta
 from numbers import Number
 
 import numpy as np
@@ -308,7 +310,9 @@ def interpolate_timeseries(data, method='linear', multivariate=False):
     for A in (X, Y):
         for i, col in enumerate(A.T):
             isnan = np.isnan(col)
-            if not isnan.any():
+            # there is no need to interpolate if there are no nans
+            # there needs to be at least two numbers
+            if not isnan.any() or sum(~isnan) < 2:
                 continue
 
             # Mean interpolation
@@ -384,7 +388,7 @@ def seasonal_decompose(data, model='multiplicative', period=12, *, callback=None
     X = []
     recomposition = sub if model == 'additive' else truediv
     interp_data = data.interp()
-    for var in data.domain:
+    for var in data.domain.variables:
         decomposed = sm.tsa.seasonal_decompose(np.ravel(interp_data[:, var]),
                                                model=model,
                                                freq=period)
@@ -447,7 +451,7 @@ def granger_causality(data, max_lag=10, alpha=.05, *, callback=None):
     # http://statsmodels.sourceforge.net/devel/vector_ar.html#granger-causality
 
     data = data.interp()
-    domain = [var for var in data.domain if var.is_continuous]
+    domain = [var for var in data.domain.variables if var.is_continuous]
     res = []
 
     for row_attr in domain:
@@ -624,3 +628,25 @@ def model_evaluation(data, models, n_folds, forecast_steps, *, callback=None):
         row[0] = row[0] + ' (in-sample)'
         res.append(row)
     return res
+
+
+SECONDS = 126230400
+DAYS = 1461
+
+
+def timestamp(dt):
+    try:
+        ts = dt.timestamp()
+    except OverflowError:
+        k = (1970 - dt.year) / 4 + 1
+        ts = (dt + timedelta(days=k * DAYS)).timestamp() - k * SECONDS
+    return ts
+
+
+def fromtimestamp(ts):
+    try:
+        dt = datetime.datetime.fromtimestamp(ts)
+    except OSError:
+        k = -ts // SECONDS + 1
+        dt = datetime.datetime.fromtimestamp(ts + k * SECONDS) - timedelta(days=k * DAYS)
+    return dt
